using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Auxiliary
{
  class FileHelper
  {
    // Extract the file extension
    public static String getSuffix(String inputFilePath)
    {
      return inputFilePath.Substring(inputFilePath.LastIndexOf('.') + 1, 3);
    }

    // Remove the file extension
    public static String stripSuffix(String inputFilePath)
    {
      return inputFilePath.Substring(0, inputFilePath.LastIndexOf('.'));
    }

    public static String openFolder()
    {
      FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
      folderBrowserDialog.RootFolder = System.Environment.SpecialFolder.MyComputer;
      if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
          return folderBrowserDialog.SelectedPath;
      else
          return "";
    }

    public static String openFolder(String directoryPath)
    {
      FolderBrowserDialog folderBrowserDialog = new FolderBrowserDialog();
      folderBrowserDialog.SelectedPath = directoryPath;
      //folderBrowserDialog.RootFolder = System.Environment.SpecialFolder.MyComputer;
      if (folderBrowserDialog.ShowDialog() == DialogResult.OK)
        return folderBrowserDialog.SelectedPath;
      else
        return "";
    }

    public static String[] getFileList(String directoryPath)
    {
      if (System.IO.Directory.Exists(directoryPath))
        return System.IO.Directory.GetFiles(directoryPath);
      else
        return new String[0];
    }

    // Show a dialog to open a file with a defined file mask
    public static String openFileDialog(String fileMask)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = fileMask;
      if (openFileDialog.ShowDialog() == DialogResult.OK)
        return openFileDialog.FileName;
      else
        return "";
    }

    // Show a dialog to open a file with a defined file mask and an initial path
    public static String openFileDialog(String fileMask, String initDir)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = fileMask;
      if (System.IO.Directory.Exists(initDir))
        openFileDialog.InitialDirectory = initDir;
      if (openFileDialog.ShowDialog() == DialogResult.OK)
        return openFileDialog.FileName;
      else
        return "";
    }

    // Show a dialog to open one or more files with a defined file mask
    public static String[] openMultipleFilesDialog(String fileMask)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = fileMask;
      openFileDialog.Multiselect = true;
      if (openFileDialog.ShowDialog() == DialogResult.OK)
        return openFileDialog.FileNames;
      else
        return new String[0];
    }

    // Show a dialog to open one or more files with a defined file mask and an initial path
    public static String[] openMultipleFilesDialog(String fileMask, String initDir)
    {
      OpenFileDialog openFileDialog = new OpenFileDialog();
      openFileDialog.Filter = fileMask;
      openFileDialog.Multiselect = true;
      if (System.IO.Directory.Exists(initDir))
        openFileDialog.InitialDirectory = initDir;
      if (openFileDialog.ShowDialog() == DialogResult.OK)
        return openFileDialog.FileNames;
      else
        return new String[0];
    }

    // Show a dialog to save a file with a defined file mask
    public static String saveFileDialog(String fileMask)
    {
      SaveFileDialog saveFileDialog = new SaveFileDialog();
      saveFileDialog.Filter = fileMask;
      if (saveFileDialog.ShowDialog() == DialogResult.OK)
        return saveFileDialog.FileName;
      else
        return "";
    }
  }
}
