﻿/************************************************************************
    STEMMER IMAGING GmbH
 ------------------------------------------------------------------------
  Program  :  LMIGocatorGenTLExample.exe
  Autor    :  L.Burzan
  Date     :  04.02.2013
  Purpose  :  Example which shows the usage of the LMI Gocator GenTL
 * It takes the RGB image from the gocator, extracts all information and displays it.
 * It is possible to save rangemap, intensitymap, stamp data and cloud of points.
 
  Secondary Purpose:
 

 
  Uses     :  cvDisplay OCX, cvImage OCX, Core3D, Metric3D, Auxilary classes from MSchatzl
  Hints    :  

  Revision :  0.0.1
  Updates  :  
 ************************************************************************/

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Cvb;

namespace LMIGocatorGenTLExample
{
  public partial class MainForm : Form
  {

  #region Members

    //String array with all stamp information as text
    private string[]  m_stampInformation = new string[14]
    {
      "Version",
      "FrameCount",
      "Timestamp",
      "EncoderValue",
      "EncoderTicks",
      "DigitalInputStates",
      "XOffset",
      "XResolution",
      "YOffset",
      "YResolution",
      "ZOffset",
      "ZResolution",
      "HightMapWidth",
      "HightMapLength",
    };


    //Cloud of points which will be displayed in COP3dViewer
    Cvb.Triangulation.COP3D m_cop;

    //Rangemap of the RGB image
    Cvb.SharedImg m_rangemap;

    //Intensitymap of the RGB image
    Cvb.SharedImg m_intensity;

    //Dataplane from the RGB image
    Cvb.SharedImg m_dataImage;

    //Stampdate which will be extracted from the data plane.
    Int64[] m_stampData;

    bool m_CVCError = false;

    #endregion

    public MainForm()
    {
      InitializeComponent();
    }

    private void MainForm_Shown(object sender, EventArgs e)
    {
      ChangeStatusText("Application loaded successful.", false);

      for (int i = 0; i < 4; i++)
        ClbSave.SetItemCheckState(i, CheckState.Checked);
    }

    //Load .VIN driver or image (be aware that this program just works with LMI Gocator images)
    private void BtnLoadDriver_Click(object sender, EventArgs e)
    {
      ChangeStatusText("Opening \"Load Image\" file dialog. Please be aware that this program just works with LMI GenTL RGB images.", false);
      m_CVCError = false;
      bool res = cvDriver.LoadImageByDialog();

      

      if (!res)
      {
        ChangeStatusText("No image loaded or acquired!", true);
      }
      else
      {
        if (!m_CVCError)
        {
          cvDriver_ImageSnaped(sender, e);
          ChangeStatusText("Image Loaded!", false); 
          GBoxCOP.Enabled = true;
        }

        if (cvDriver.IsGrabber)
        {
          BtnSnap.Enabled = true;
        }
        else
        {
          BtnSnap.Enabled = false;
        }


      }
    }

    //Change status text to a given text.
    //Background set to RED if error == true
    private void ChangeStatusText(string text, bool error)
    {
      if (error)
      {
        StatusBarMainForm.BackColor = ControlPaint.Light(Color.Red, 95f);
      }
      else
      {
        StatusBarMainForm.BackColor = ControlPaint.Light(Color.Green, 95f);
      }
      StatusMainForm.Text = text;
      AddStatusTextToRichBox(text);
    }

    //Write text output into log status text box.
    private void AddStatusTextToRichBox(string text)
    {
      if (richTextBox1.Text.Length == 0)
        BtnSaveLog.Enabled = true;

      string tmpText = richTextBox1.Text;
      richTextBox1.Text = tmpText + text + "\r\n";

      richTextBox1.SelectionStart = richTextBox1.Text.Length;
      richTextBox1.ScrollToCaret();
    }

    //Image snaped event. 
    //Extract data / Display Intensitymap and Rangemap / Create cop and show cop into cop 3D viewer
    private void cvDriver_ImageSnaped(object sender, EventArgs e)
    {

      //Extract Stamp data.
      ExtractData();

      //Get X resolution from stamp data
      TbXResolution.Text = m_stampData[07].ToString();

      //Get Y resolution from stamp data
      TbYResolution.Text = m_stampData[09].ToString();

      //Get Z resolution from stamp data
      TbZResolution.Text = m_stampData[11].ToString();

      if (!GBoxImage.Enabled)
        GBoxImage.Enabled = true;

      try
      {

        Cvb.Triangulation.Metric3DFactors metric = new Cvb.Triangulation.Metric3DFactors(float.Parse(TbXResolution.Text), float.Parse(TbYResolution.Text), float.Parse(TbZResolution.Text));


        if (m_cop != null)
        {
          m_cop.Dispose();
        }

        //create new metric calibrated cop
        m_cop = new Cvb.Triangulation.COP3D(m_rangemap, metric);

        //Display cop into Cop3dViewer.
        cvCOPDisplay.SetCOP3D(m_cop.Handle.ToInt32(), 0, 1);

        cvCOPDisplay.Refresh();

        if (!GBoxCOP.Enabled)
          GBoxCOP.Enabled = true;

        ChangeStatusText("Image acquired!", false);
      }
      catch(Cvb.Triangulation.Core3DException ex)
      {
        ChangeStatusText("Displaying cloud of points failed. Error Code: " + ex.ErrorCode, true);
      }



    }

    //Extract rangemap, intensitymap and data image from RGB iamge.
    private void ExtractData()
    {
      if (m_rangemap != 0)
        m_rangemap.Dispose();

      if (m_intensity != 0)
        m_intensity.Dispose();

      if (m_dataImage != 0)
        m_dataImage.Dispose();

      Cvb.Image.TArea area;

      //Extract data image
      Cvb.Image.SetArea(0, 0, 0, cvDriver.ImageHeight, cvDriver.ImageWidth, 0, out area);
      bool res = Cvb.Image.CreateGenericImageDT(1, cvDriver.ImageWidth, cvDriver.ImageHeight, 16, out m_dataImage);
      Cvb.Image.CopyImageArea(cvDriver.Image, m_dataImage, 2, 0, area, 0, 0);     


      //extract stamp data
      ExtractStampData();
      OutputStampData();

      //Extract rangemap
      Cvb.Image.CreateGenericImageDT(1, (int) m_stampData[12], (int) m_stampData[13], 16, out m_rangemap);
      Cvb.Image.CopyImageArea(cvDriver.Image, m_rangemap, 0, 0, area, 0, 0);

      //Cvb.Image.WriteImageFile(m_rangemap, "test_rangemap.tiff"); 
      cvRangemapDisplay.Image = m_rangemap;
      cvRangemapDisplay.Refresh();

      Cvb.SharedImg img;
      //Extract intensity map
      Cvb.Image.CreateGenericImageDT(1, (int) m_stampData[12], (int) m_stampData[13], 16, out img);
      Cvb.Image.CopyImageArea(cvDriver.Image, img, 1, 0, area, 0, 0);

      Cvb.Image.MapTo8Bit(img, false, out m_intensity);

      cvIntensityDisplay.Image = m_intensity;
      cvIntensityDisplay.Refresh();

    }

    //Extract stamp data from data image.
    private void ExtractStampData()
    {
      if (m_dataImage != 0)
      {
        int value;
        Int16[] stampDataUnconverted = new Int16[56];

        for(int i = 0; i < 56; i++)
        {
          //The function GetPixel normally shouldn't be used in an C# application as it is slowing down the processing very much. Please use GetLinearAccess instead.
          //In this case I used "GetPixel" to fasten up the programming as it is fairly easy to include the function. I will change this part into a GetLinearAccess as soon as I find the time.
          Cvb.Image.GetPixel(m_dataImage, 0, i, 0, out value);
          stampDataUnconverted[i] = (Int16) value;

        }
        m_stampData = ConvertStampData(stampDataUnconverted);
      }
      else
      {
        ChangeStatusText("No data image available, so no data information could be red", false);
      }
    }

    //Output stamp data into status text box
    private void OutputStampData()
    {
      if (m_stampData != null)
      {
        AddStatusTextToRichBox("Stamp Information:");
        for (int i = 0; i < 14; i++)
        {
          AddStatusTextToRichBox(m_stampInformation[i] + ": " + m_stampData[i].ToString());
        }
        AddStatusTextToRichBox("\r\n");
      }
    }

    //Convert each four 16bit stamp data pixel into one 64 bit stamp data value.
    private Int64[] ConvertStampData(Int16[] data)
    {
      Int64[] stampData = new Int64[14];
      Array.Clear(stampData, 0, stampData.Length);
      Int64 val1 = 0, val2 = 0, val3 = 0, val4 = 0;
      for (int i = 0; i < 56; i += 4)
      {
        val1 = data[i];
        val2 = data[i + 1];
        val3 = data[i + 2];
        val4 = data[i + 3];

        stampData[i / 4] = ((val1 & 0xFFFF) << 48) | ((val2 & 0xFFFF) << 32) | ((val3 & 0xFFFF) << 16) | (val4 & 0xFFFF);
      }     
      return stampData;
    }

    private void BtnSnap_Click(object sender, EventArgs e)
    {
      bool res = cvDriver.Snap();

      if (res)
      {
        cvDriver_ImageSnaped(sender, e);
      }
      else
      {
        ChangeStatusText("No image acquired!", true);
      }
    }

    private void BtnSaveRangemap_Click(object sender, EventArgs e)
    {
      string fileName = Auxiliary.FileHelper.saveFileDialog("TIF (*.tif)|*.tif");

      

      if (fileName.Length != 0)
      {
        string stripFileName = Auxiliary.FileHelper.stripSuffix(fileName);

        if (ClbSave.GetItemCheckState(0) == CheckState.Checked)
        {
          Cvb.Image.WriteImageFile(cvDriver.Image, stripFileName + "_LMIGenTLImg.tif");
          ChangeStatusText("GenTL Image saved.", false);
        }

        if (ClbSave.GetItemCheckState(1) == CheckState.Checked)
        {
        Cvb.Image.WriteImageFile(m_intensity, stripFileName + "_Intensity.tif");
        ChangeStatusText("Intesitymap saved.", false);
        }

        if (ClbSave.GetItemCheckState(2) == CheckState.Checked)
        {
        Cvb.Image.WriteImageFile(m_rangemap, stripFileName + "_Rangemap.tif");
        ChangeStatusText("Rangemap saved.", false);
        }

        if (ClbSave.GetItemCheckState(3) == CheckState.Checked)
        {
          SaveStampData(stripFileName + "_StampData.txt");
          ChangeStatusText("Stamp information saved.", false);
        }
      }
      else
      {
        ChangeStatusText("Rangemap NOT saved.", true);
      }
    }

    private void SaveStampData(string fileName)
    {
      String stampDataTxt = "Stamp data information\r\n"
        + "Version: " + m_stampData[0] + "\r\n"
       + "FrameCount: " + m_stampData[1] + "\r\n"
       + "Timestamp: " + m_stampData[2] + "\r\n"
       + "EncoderValue: " + m_stampData[3] + "\r\n"
       + "EncoderTicks: " + m_stampData[4] + "\r\n"
       + "DigitalInputStates: " + m_stampData[5] + "\r\n"
       + "XOffset: " + m_stampData[6] + "\r\n"
       + "XResolution: " + m_stampData[7] + "\r\n"
       + "YOffset: " + m_stampData[8] + "\r\n"
       + "YResolution: " + m_stampData[9] + "\r\n"
       + "ZOffset: " + m_stampData[10] + "\r\n"
       + "ZResolution: " + m_stampData[11] + "\r\n"
       + "HightMapWidth: " + m_stampData[12] + "\r\n"
       + "HightMapLength: " + m_stampData[13] + "\r\n";

      System.IO.File.WriteAllText(fileName, stampDataTxt);


    }

    private void checkBox1_CheckedChanged(object sender, EventArgs e)
    {
      cvDriver.Grab = !cvDriver.Grab;
    }

    private void BtnClearLog_Click(object sender, EventArgs e)
    {
      richTextBox1.Clear();
      BtnSaveLog.Enabled = false;
    }

    private void BtnSaveLog_Click(object sender, EventArgs e)
    {
     
      string fileName = Auxiliary.FileHelper.saveFileDialog("Text File (*.txt)|*.txt");

      if (fileName.Length != 0)
      {
        System.IO.File.WriteAllText(fileName, richTextBox1.Text);
      }
      else
      {
        ChangeStatusText("No log file was saved.", true);
      }
    }

    private void BtnReset_Click(object sender, EventArgs e)
    {
      cvCOPDisplay.ResetView();
    }

    private void BtnSaveCop_Click(object sender, EventArgs e)
    {
      ChangeStatusText("Saving cloud of points to file..", false);

      //Open a save file dialog
      string fileName = Auxiliary.FileHelper.saveFileDialog("Binary Polygon File (*.ply)|*.ply|Cloud Of Points File (*.cop)|*.cop|Point Cloud Data File (*.pcd)|*.pcd|Ascii Point List File (*.asc)|*.asc");


      if (fileName.Length != 0)
      {
        //Get the chosen file extension
        string extension = System.IO.Path.GetExtension(fileName);

        string text = "";

        string auxilaryFileFormat = "";

        Cvb.Triangulation.COP3DFileFormat format = 0;

        switch (extension)
        {

          case ".ply":
            format = Cvb.Triangulation.COP3DFileFormat.PLYBinary;
            text = "Saved binary polygon (*.ply) file..";
            break;
          case ".cop":
            format = Cvb.Triangulation.COP3DFileFormat.COP3D;
            text = "Saved cloud of points (*.cop) file..";
            break;
          case ".pcd":
            auxilaryFileFormat = "pcd";
            text = "Saved point list data (*.pcd) file..";
            break;
          case ".asc":
            auxilaryFileFormat = "asc";
            text = "Saved ASCII point list (*.asc) file..";
            break;
          default:
            ChangeStatusText("Selected Index unkown!", true);
            fileName = "Error";
            break;
        }

        if (fileName.CompareTo("Error") != 0 && auxilaryFileFormat.Length == 0)
        {
          try
          {
            //Save cloud of points to a specific file name and format
            m_cop.Save(fileName, format);
            ChangeStatusText(text, false);
          }
          catch (Cvb.Triangulation.Core3DException ex)
          {
            ChangeStatusText("Saving calibrated cloud of points failed. Error Code: " + ex.ErrorCode, true);
          }
        }
        else if (fileName.CompareTo("Error") != 0)
        {
          Cvb.Image.IMG img = m_cop.Image;
          switch (auxilaryFileFormat)
          {
            case "pcd":
              Toolbox3D.FileIO.PointCloudData.Write(ref img, fileName);
              ChangeStatusText(text, false);
              break;
            case "asc":
              Toolbox3D.FileIO.AsciiPointList.Write(ref img, fileName);
              ChangeStatusText(text, false);
              break;
            default: ChangeStatusText("Error saving PCD or ASC file.", true);
              break;
          }
        }
      }
      else
      {
        ChangeStatusText("No cloud of points data saved to file.", true);
      }
    }

    private void StatusMainForm_TextChanged(object sender, EventArgs e)
    {
      StatusBarMainForm.Refresh();
    }

    private void cvDriver_CVCError(object sender, AxCVIMAGELib._DCVimageEvents_CVCErrorEvent e)
    {
      ChangeStatusText("CVCError: " + e.errorString, true);
      m_CVCError = true;
      
    }

    private void cvRangemapDisplay_ClickEvent(object sender, EventArgs e)
    {
        Point position = cvRangemapDisplay.PointToClient(Cursor.Position);

        int X = 0;
        int Y = 0;

        bool res = cvRangemapDisplay.ClientToImage(position.X, position.Y, ref X, ref Y);

        if (!res)
        {
          ChangeStatusText("Extracting coordinates failed.", true);
        }
        else
        {
          AddLabelToCvRangemapDisplay(X, Y);
        }
    }

    private void cvIntensityDisplay_ClickEvent(object sender, EventArgs e)
    {
      Point position = cvIntensityDisplay.PointToClient(Cursor.Position);

      int X = 0;
      int Y = 0;

      bool res = cvIntensityDisplay.ClientToImage(position.X, position.Y, ref X, ref Y);

      if (!res)
      {
        ChangeStatusText("Extracting coordinates failed.", true);
      }
      else
      {
        AddLabelToCvRangemapDisplay(X, Y);
      }
    }

    private void AddLabelToCvRangemapDisplay(int X, int Y)
    {
      //m_stampData[6] = XOffset & m_stampData[7] == XResolution
      double metricX = (m_stampData[6] + X * m_stampData[7]) / 1000d;

      //m_stampData[8] == YOffset & m_stampData[9] == YResolution
      double metricY = (m_stampData[8] + Y * m_stampData[9]) / 1000d;

      int pixelValue = 0;
      bool res = Cvb.Image.GetPixel(m_rangemap, 0, X, Y, out pixelValue);

      double metricZ = 0;
      if (res)
      {
        if (pixelValue == 0)
        {
          metricZ = 0;
        }
        else
        {
          //m_stampData[10] == ZOffset & m_stampData[11] == ZResolution
          metricZ = (m_stampData[10] + pixelValue * m_stampData[11]) / 1000d;
        }
      }
      else
      {
        ChangeStatusText("Could not read Pixelvalue.", true);
      }

      string text = "X: " + metricX.ToString() + " mm, "
                      + "Y: " + metricY.ToString() + " mm, "
                      + "Z: " + metricZ.ToString() + " mm. ";

      cvRangemapDisplay.RemoveAllLabels();

      int color = int.Parse("00ffff", System.Globalization.NumberStyles.AllowHexSpecifier);
      cvRangemapDisplay.AddLabel(text, false,  color, 0, X, Y);

      ChangeStatusText(text + " at  pixel (" + X + ", " + Y + ")", false);
    }






  }
}
